Math.randomBetween = function(a, b)
{
	return (Math.random() * (b-a)) + a;
};

Vector.fromAngle = function(a, l)
{
	return new Vector(l * Math.cos(a), l * Math.sin(a));
};

Vector.prototype.print = function()
{
	print(this);
};

Vector.prototype.toString = function()
{
	return 'Vector: ('+this.x+', '+this.y+')';
};

Vector.prototype.subtract = function(v)
{
	return new Vector(this.x - v.x, this.y - v.y);
};

Vector.prototype.add = function(v)
{
	return new Vector(this.x + v.x, this.y + v.y);
};

Vector.prototype.scalar = function(s)
{
	return new Vector(this.x * s, this.y * s);
};

Vector.prototype.length = function(v)
{
	return Math.sqrt(this.x * this.x + this.y * this.y);
};

Vector.prototype.angle = function(v)
{
	return Math.atan2(this.y, this.x).angle();
};

Vector.prototype.equals = function(v)
{
	return (Math.abs(this.x - v.x) < Math.pow(10, -5) && Math.abs(this.y - v.y) < Math.pow(10, -5));
};

Vector.prototype.unit = function(v)
{
	return Vector.fromAngle(this.angle(), 1);
};

Vector.prototype.dot = function(v)
{
	return this.x*v.x + this.y*v.y;
};

Number.prototype.degrees = function()
{
	return 180 * this.valueOf().angle() / Math.PI;
}

Number.prototype.radians = function()
{
	return (Math.PI * this.valueOf() / 180).angle();
}

Number.prototype.angle = function()
{
	var a = this.valueOf();
	
	while(a < -Math.PI)
		a += 2 * Math.PI;
	while(a >= Math.PI)
		a -= 2 * Math.PI;
		
	return a;
};


Number.prototype.clamp = function(a, b)
{
	return Math.max(Math.min(this.valueOf(), Math.max(a, b)), Math.min(a, b));
};

Number.prototype.round = function()
{
	return Math.round(this.valueOf());
};

Number.prototype.floor = function()
{
	return Math.floor(this.valueOf());
};

Number.prototype.ceil = function()
{
	return Math.ceil(this.valueOf());
};

Number.prototype.sign = function()
{
	return (this.valueOf() >= 0)?1:-1;
};

Number.prototype.abs = function()
{
	return Math.abs(this.valueOf());
};

SensedRobot.prototype.print = function()
{
	print('Sensed robot: ' + this.id);
};

SensedMunition.prototype.print = function()
{
	print('Sensed munition: ' + this.id);
};

Array.prototype.foreach = function(fn)
{
	for(var i = 0; i < this.length; ++i)
		fn.apply(this[i], []);
};

Array.prototype.transform = function(fn)
{
	var x = [];
	
	for(var i = 0; i < this.length; ++i)
		x.push(fn.apply(this[i], []));
		
	return x;
};

Array.prototype.isEmpty = function()
{
	return (this.length === 0);
};

Array.prototype.peek = function()
{
	return (this[this.length-1]);
};

Array.prototype.contains = function(x)
{
	return (this.indexOf(x) >= 0);
};

Array.prototype.removeIf = function(fn)
{
	var x = [];
	
	for(var i = 0; i < this.length; ++i)
		if(!fn.apply(this[i], []))
			x.push(this[i]);
		
	return x;
};

String.prototype.times = function(n)
{
	var s = [];
	for(var i = 0; i < n; ++i)
		s.push(this);
	return s.join('');
};

function inherit(d, b)
{
	for(var key in b.prototype)
		d.prototype[key] = b.prototype[key];
}
